@@ -11,10 +11,15 @@
 
 -include_lib("eunit/include/eunit.hrl").
 
+
+run() ->
+    eunit:test([ts_test_all], [{report,{eunit_surefire,[{dir,"."}]}}]).
+
 test() -> ok.
 
 all_test_() -> [ts_test_recorder,
                 ts_test_config,
+                ts_test_client,
                 ts_test_dynvars_api,
                 ts_test_file_server,
                 ts_test_options,
@@ -26,4 +31,9 @@ all_test_() -> [ts_test_recorder,
                 ts_test_mon,
                 ts_test_user_server,
                 ts_test_search,
-                ts_test_stats].
+                ts_test_stats,
+                ts_test_interaction,
+                ts_test_websocket,
+                ts_test_utils,
+                ts_test_mqtt
+               ].
