@@ -36,7 +36,7 @@ parse_dyn_var_jsonpath2_test() ->
     myset_env(),
     Data="\r\n\r\n{\"titi\": [23,45]}",
     JSONPath = "titi[3]",
-    ?assertEqual([{'myvar',undefined}], ts_search:parse_dynvar([{jsonpath,'myvar', JSONPath} ],list_to_binary(Data))).
+    ?assertEqual([{'myvar',<< >>}], ts_search:parse_dynvar([{jsonpath,'myvar', JSONPath} ],list_to_binary(Data))).
 
 parse_dyn_var_jsonpath3_test() ->
     myset_env(),
@@ -48,7 +48,7 @@ parse_dyn_var_jsonpath4_test() ->
     myset_env(),
     Data="\r\n\r\n{\"titi\": [{\"val\": 123, \"name\": \"foo\"}, {\"val\": 42, \"name\": \"bar\"}]}",
     JSONPath = "titi[?name=void].val",
-    ?assertEqual([{'myvar',undefined}], ts_search:parse_dynvar([{jsonpath,'myvar', JSONPath} ],list_to_binary(Data))).
+    ?assertEqual([{'myvar', << >>}], ts_search:parse_dynvar([{jsonpath,'myvar', JSONPath} ],list_to_binary(Data))).
 
 parse_dyn_var_jsonpath5_test() ->
     myset_env(),
@@ -74,6 +74,13 @@ parse_dyn_var_jsonpath_xmpp_test() ->
     JSONPath = "nodes",
     ?assertMatch([{'nodes',[<<"suno-12">>,<<"suno-13">>]}], ts_search:parse_dynvar([{jsonpath,'nodes', JSONPath} ],list_to_binary(Data))).
 
+parse_dyn_var_jsonpath_struct_test() ->
+    myset_env(),
+    Data="{\"accessToken\":{\"id\":\"78548a96-cadd-48c0-b7d6-4ff3b81f10cc\",\"lists\":[\"testlist1\"],\"token\":\"rTgdC3f7uJ/Smg3s4b9va2KW5GdPkRHtwYNgWbvwhensgOSf2/wan95VPDiXKnAAGilsZlpw/Td4bs/OPeVeYg==\",\"scope\":[\"GET_ME\",\"WRITE_ACCESS\"]},\"accessTokenSignature\":\"gWAL+zvDcQjqLmNdSwcG/TOWyta5g==\"}",
+    JSONPath = "accessToken",
+    ?assertMatch([{'nodes',  << "{\"id\":\"78548a96-cadd-48c0-b7d6-4ff3b81f10cc\",\"lists\":[\"testlist1\"],\"token\":\"rTgdC3f7uJ/Smg3s4b9va2KW5GdPkRHtwYNgWbvwhensgOSf2/wan95VPDiXKnAAGilsZlpw/Td4bs/OPeVeYg==\",\"scope\":[\"GET_ME\",\"WRITE_ACCESS\"]}" >> }], ts_search:parse_dynvar([{jsonpath,'nodes', JSONPath} ],list_to_binary(Data))).
+
+
 parse_dyn_var_xpath_test() ->
     myset_env(),
     Data="\r\n\r\n<html><body>"++?FORMDATA++"</body></html>",
@@ -203,6 +210,14 @@ format_result(Data,binary) ->
 format_result(Data,_) ->
     Data.
 
+parse_re_decode_test() ->
+    myset_env(),
+    Data= << "<input name=\"jsf_tree_64\" value=\"&apos;foo&amp;bar&apos;\">" >>,
+    StrName="jsf_tree_64",
+    Regexp = ?DEF_RE_DYNVAR_BEGIN++ StrName ++?DEF_RE_DYNVAR_END,%'
+    [{Name,Value}] = ts_search:parse_dynvar([{re, 'jsf_tree_64', Regexp, fun ts_utils:conv_entities/1 }],Data),
+    ?assertEqual("'foo&bar'", ts_search:subst("%%_jsf_tree_64%%",[{Name,Value}])).
+
 parse_subst1_re_test() ->
     myset_env(),
     Data=?FORMDATA,
@@ -265,32 +280,71 @@ parse_subst_regexp_sid_test() ->
 
 dynvars_urandom_test() ->
     myset_env(),
-    ?assertMatch(["qxvmvtglimieyhemzlxc"],ts_client:set_dynvars(urandom,{string,20},[toto],[])).
+    ?assertMatch([<<"qxvmvtglimieyhemzlxc">>],ts_client:set_dynvars(urandom,{string,20},[toto],[],{},[])).
 
 dynvars_urandom_neg_test() ->
     myset_env(),
-    ?assertError(function_clause,ts_client:set_dynvars(urandom,{string,-3},[toto],[])).
+    ?assertError(function_clause,ts_client:set_dynvars(urandom,{string,-3},[toto],[],{},[])).
 
 dynvars_urandom2_test() ->
     myset_env(),
-    ?assertMatch(["qxvmvtglimieyhemzlxc","qxvmvtglimieyhemzlxc"],ts_client:set_dynvars(urandom,{string,20},[toto,tutu],[])).
+    ?assertMatch([<<"qxvmvtglimieyhemzlxc">>,<<"qxvmvtglimieyhemzlxc">>],ts_client:set_dynvars(urandom,{string,20},[toto,tutu],[],{},[])).
 
 dynvars_random_test() ->
     myset_env(),
-    [String] = ts_client:set_dynvars(random,{string,20},[toto],[]),
-    ?assertMatch(20,length(String)).
+    [String] = ts_client:set_dynvars(random,{string,20},[toto],[],{},[]),
+    ?assertMatch(20,length(binary_to_list(String))).
 
 dynvars_random2_test() ->
     myset_env(),
-    [String,String2] = ts_client:set_dynvars(random,{string,20},[toto,titi],[]),
-    ?assertMatch({20,20},{length(String),length(String2)}).
+    [String,String2] = ts_client:set_dynvars(random,{string,20},[toto,titi],[],{},[]),
+    ?assertMatch({20,20},{length(binary_to_list(String)),length(binary_to_list(String2))}).
 
 dynvars_jsonpath_test() ->
     myset_env(),
     Data="\r\n\r\n{\"titi\": [{\"val\": 123, \"name\": \"foo\"}, {\"val\": 42, \"name\": \"bar\"}]}",
     JSONPath = "titi[?name=bar].val",
     Dynvars=ts_dynvars:new(data,Data),
-    ?assertEqual(42,ts_client:set_dynvars(jsonpath,{JSONPath,data},[toto],#dyndata{dynvars=Dynvars})).
+    ?assertEqual(42,ts_client:set_dynvars(jsonpath,{JSONPath,data},[toto],Dynvars,{},[])).
+
+dynvars_jsonpath2_test() ->
+    myset_env(),
+    Data="{\"accessToken\":{\"id\":\"78548a96-cadd-48c0-b7d6-4ff3b81f10cc\",\"lists\":[\"testlist1\"],\"token\":\"rTgdC3f7uJ/Smg3s4b9va2KW5GdPkRHtwYNgWbvwhensgOSf2/wan95VPDiXKnAAGilsZlpw/Td4bs/OPeVeYg==\",\"scope\":[\"GET_ME\",\"WRITE_ACCESS\"]},\"accessTokenSignature\":\"gWAL+zvDcQjqLmNdSwcG/TOWyta5g==\"}",
+    JSONPath = "accessToken",
+    JSONPath2 = "accessTokenSignature",
+    Dynvars=ts_dynvars:new(data,Data),
+    Res = << "{\"id\":\"78548a96-cadd-48c0-b7d6-4ff3b81f10cc\",\"lists\":[\"testlist1\"],\"token\":\"rTgdC3f7uJ/Smg3s4b9va2KW5GdPkRHtwYNgWbvwhensgOSf2/wan95VPDiXKnAAGilsZlpw/Td4bs/OPeVeYg==\",\"scope\":[\"GET_ME\",\"WRITE_ACCESS\"]}" >>,
+    ?assertEqual(Res,ts_client:set_dynvars(jsonpath,{JSONPath,data},[toto],Dynvars,{},[])),
+    ?assertEqual(<< "gWAL+zvDcQjqLmNdSwcG/TOWyta5g==" >>,ts_client:set_dynvars(jsonpath,{JSONPath2,data},[toto],Dynvars,{},[])).
+
+dynvars_jsonpath3_test() ->
+    myset_env(),
+    Data="\r\n\r\n{\"titi\": [42, 666] }",
+    JSONPath = "titi[1]",
+    Dynvars=ts_dynvars:new(data,Data),
+    ?assertEqual(666,ts_client:set_dynvars(jsonpath,{JSONPath,data},[toto],Dynvars,{},[])).
+
+dynvars_jsonpath4_test() ->
+    myset_env(),
+    Data="\r\n\r\n{\"titi\": [{\"val\": [ 123, 231 ] , \"name\": \"foo\"}, {\"val\":  [ 13, 23 ], \"name\": \"bar\"}]}",
+    JSONPath = "titi[?name=bar].val[0]",
+    Dynvars=ts_dynvars:new(data,Data),
+    ?assertEqual(13,ts_client:set_dynvars(jsonpath,{JSONPath,data},[toto],Dynvars,{},[])).
+
+dynvars_file_test() ->
+    myset_env(),
+    ts_file_server:stop(),
+    ts_file_server:start(),
+    ts_file_server:read([{default,"./src/test/test_file_server.csv"}]),
+    ?assertMatch([<<"username1">>,<<"glop">>, << >>],ts_client:set_dynvars(file,{iter,default,<< ";" >>},[],[],{},[])).
+
+dynvars_file_pipe_test() ->
+    myset_env(),
+    ts_file_server:stop(),
+    ts_file_server:start(),
+    ts_file_server:read([{default,"./src/test/test_file_server_pipe.csv"}]),
+    ?assertMatch([<<"conv%2F99%2F589%2Finfo.txt">>,<<"99">> ,<<"589">>],ts_client:set_dynvars(file,{iter,default,<< "|" >>},[],[],{},[])).
+
 
 
 %%TODO: out of order..
