@@ -21,7 +21,8 @@
 %%%  In addition, as a special exception, you have the permission to
 %%%  link the code of this program with any library released under
 %%%  the EPL license and distribute linked combinations including
-%%%  the two.
+%%%  the two; the MPL (Mozilla Public License), which EPL (Erlang
+%%%  Public License) is based on, is included in this exception.
 
 -module(ts_fs).
 -vc('$Id: ts_erlang.erl,v 0.0 2009/08/20 16:31:58 nniclaus Exp $ ').
@@ -29,12 +30,12 @@
 
 -behavior(ts_plugin).
 
+-include("ts_macros.hrl").
 -include("ts_profile.hrl").
 -include("ts_fs.hrl").
 -include_lib("kernel/include/file.hrl").
 
--export([init_dynparams/0,
-         add_dynparams/4,
+-export([add_dynparams/4,
          get_message/2,
          session_defaults/0,
          dump/2,
@@ -93,35 +94,30 @@ session_defaults() ->
 %% @doc Initialises the state for a new protocol session.
 %% @end
 new_session() ->
-    #fs{}.
+    #fs_session{}.
 
 %% @spec decode_buffer(Buffer::binary(),Session::record(fs)) ->  NewBuffer::binary()
 %% @doc We need to decode buffer (remove chunks, decompress ...) for
 %%      matching or dyn_variables
 %% @end
-decode_buffer(Buffer,#fs{}) ->
+decode_buffer(Buffer,#fs_session{}) ->
     Buffer.
 
-%% @spec init_dynparams() -> dyndata()
-%% @doc Creates a new record/term for storing dynamic request data.
-%% @end
-init_dynparams() ->
-    #dyndata{proto=#fs_dyndata{}}.
 
 %% @spec add_dynparams(Subst, dyndata(), param(), hostdata()) -> {dyndata(), server()} | dyndata()
 %% Subst = term()
 %% @doc Updates the dynamic request data structure created by
 %% {@link ts_protocol:init_dynparams/0. init_dynparams/0}.
 %% @end
-add_dynparams(false, DynData, Param, HostData) ->
-    add_dynparams(DynData#dyndata.proto, Param, HostData);
-add_dynparams(true, DynData, Param, HostData) ->
-    NewParam = subst(Param, DynData#dyndata.dynvars),
-    add_dynparams(DynData#dyndata.proto,NewParam, HostData).
+add_dynparams(false, {_,Session}, Param, HostData) ->
+    add_dynparams(Session, Param, HostData);
+add_dynparams(true, {DynVars,Session}, Param, HostData) ->
+    NewParam = subst(Param, DynVars),
+    add_dynparams(Session,NewParam, HostData).
 
-add_dynparams(#fs_dyndata{position=Pos,iodev=IODevice}, Req=#fs{}, _HostData) when is_integer(Pos)->
+add_dynparams(#fs_session{position=Pos,iodev=IODevice}, Req=#fs{}, _HostData) when is_integer(Pos)->
     Req#fs{position=Pos,iodev=IODevice};
-add_dynparams(#fs_dyndata{}, Param, _HostData) ->
+add_dynparams(#fs_session{}, Param, _HostData) ->
     Param.
 
 %%----------------------------------------------------------------------
@@ -129,8 +125,8 @@ add_dynparams(#fs_dyndata{}, Param, _HostData) ->
 %% @doc Replace on the fly dynamic element of the request.
 %% @end
 %%----------------------------------------------------------------------
-subst(Req=#fs{path=Path,size=Size}, DynVars) ->
-    Req#fs{path=ts_search:subst(Path,DynVars),size=ts_search:subst(Size,DynVars)}.
+subst(Req=#fs{path=Path,size=Size,dest=Dest}, DynVars) ->
+    Req#fs{path=ts_search:subst(Path,DynVars),dest=ts_search:subst(Dest,DynVars), size=ts_search:subst(Size,DynVars)}.
 
 %% @spec parse(Data::client_data(), State) -> {NewState, Opts, Close}
 %% State = #state_rcv{}
@@ -143,27 +139,27 @@ subst(Req=#fs{path=Path,size=Size}, DynVars) ->
 %% Setting Close to true will cause tsung to close the connection to
 %% the server.
 %% @end
-parse({file, open, _Args, {ok,IODevice}},State=#state_rcv{dyndata=DynData}) ->
-    NewDyn=(DynData#dyndata.proto)#fs_dyndata{iodev=IODevice,position=0},
-    {State#state_rcv{ack_done=true,datasize=0,dyndata=DynData#dyndata{proto=NewDyn}}, [], false};
+parse({file, open, _Args, {ok,IODevice}},State=#state_rcv{session=S}) ->
+    NewDyn=S#fs_session{iodev=IODevice,position=0},
+    {State#state_rcv{ack_done=true,datasize=0,session=NewDyn}, [], false};
 parse({file, open, [Path,_], {error,Reason}},State) ->
     ?LOGF("error while opening file: ~p(~p)~n",[Path, Reason],?ERR),
     ts_mon:add({count,error_fs_open}),
     {State#state_rcv{ack_done=true,datasize=0}, [], false};
-parse({file, close, [_IODevice], ok},State=#state_rcv{dyndata=DynData}) ->
-    NewDyn=(DynData#dyndata.proto)#fs_dyndata{iodev=undefined,position=0},
-    {State#state_rcv{ack_done=true,datasize=0,dyndata=DynData#dyndata{proto=NewDyn}}, [], false};
+parse({file, close, [_IODevice], ok},State=#state_rcv{session=S}) ->
+    NewDyn=S#fs_session{iodev=undefined,position=0},
+    {State#state_rcv{ack_done=true,datasize=0,session=NewDyn}, [], false};
 parse({file, close, [_IODevice], {error,Reason}}, State) ->
     ?LOGF("error while closing file: ~p~n",[Reason],?ERR),
     ts_mon:add({count,error_fs_close}),
     {State#state_rcv{ack_done=true,datasize=0}, [], false};
 
-parse({file, pread, [_IODev,Pos,Size], {ok,_Data}},State=#state_rcv{dyndata=DynData,datasize=DataSize}) ->
-    NewDyn=(DynData#dyndata.proto)#fs_dyndata{position=Pos+Size},
-    {State#state_rcv{ack_done=true,datasize=DataSize+Size,dyndata=DynData#dyndata{proto=NewDyn}}, [], false};
-parse({file, pread, [_IODev,_Pos,Size], eof},State=#state_rcv{dyndata=DynData,datasize=DataSize}) ->
-    NewDyn=(DynData#dyndata.proto)#fs_dyndata{position=0},
-    {State#state_rcv{ack_done=true,datasize=DataSize+Size,dyndata=DynData#dyndata{proto=NewDyn}}, [], false};
+parse({file, pread, [_IODev,Pos,Size], {ok,_Data}},State=#state_rcv{session=S,datasize=DataSize}) ->
+    NewDyn=S#fs_session{position=Pos+Size},
+    {State#state_rcv{ack_done=true,datasize=DataSize+Size,session=NewDyn}, [], false};
+parse({file, pread, [_IODev,_Pos,Size], eof},State=#state_rcv{session=S,datasize=DataSize}) ->
+    NewDyn=S#fs_session{position=0},
+    {State#state_rcv{ack_done=true,datasize=DataSize+Size,session=NewDyn}, [], false};
 parse({file, pread, [_IODev,_Pos,_Size], {error,Reason}},State) ->
     ?LOGF("error while reading file: ~p~n",[Reason],?ERR),
     ts_mon:add({count,error_fs_pread}),
@@ -175,9 +171,13 @@ parse({file, write_file, [Path,_], {error,Reason}},State) ->
     ?LOGF("error while writing file: ~p (~p)~n",[Path, Reason],?ERR),
     ts_mon:add({count,error_fs_write}),
     {State#state_rcv{ack_done=true, datasize=0}, [], false};
-parse({file, pwrite, [_IODev,Pos,Data], ok},State=#state_rcv{dyndata=DynData}) ->
-    NewDyn=(DynData#dyndata.proto)#fs_dyndata{position=Pos+length(Data)},
-    {State#state_rcv{ack_done=true,datasize=0,dyndata=DynData#dyndata{proto=NewDyn}}, [], false};
+parse({file, pwrite, [_IODev,Pos,Data], ok},State=#state_rcv{session=S}) ->
+    NewDyn=S#fs_session{position=Pos+length(Data)},
+    {State#state_rcv{ack_done=true,datasize=0,session=NewDyn}, [], false};
+parse({file, pwrite, Args, {error,Reason}},State) ->
+    ?LOGF("error while writing file: ~p (~p)~n",[Args, Reason],?ERR),
+    ts_mon:add({count,error_fs_pwrite}),
+    {State#state_rcv{ack_done=true, datasize=0}, [], false};
 
 parse({file, del_dir, [_Path], ok},State) ->
     {State#state_rcv{ack_done=true, datasize=0}, [], false};
@@ -185,18 +185,41 @@ parse({file, del_dir, [Path], {error,Reason}},State) ->
     ?LOGF("error while delete directory: ~p (~p)~n",[Path, Reason],?ERR),
     ts_mon:add({count,error_fs_del_dir}),
     {State#state_rcv{ack_done=true, datasize=0}, [], false};
+
 parse({file, make_dir, [_Path], ok},State) ->
     {State#state_rcv{ack_done=true, datasize=0}, [], false};
+parse({file, make_dir, [Path], {error, eexist} },State) ->
+    ?LOGF("error while creating diretory: ~p already exists~n",[Path],?NOTICE),
+    {State#state_rcv{ack_done=true, datasize=0}, [], false};
 parse({file, make_dir, [Path], {error,Reason}},State) ->
     ?LOGF("error while creating diretory: ~p (~p)~n",[Path, Reason],?ERR),
     ts_mon:add({count,error_fs_mkdir}),
     {State#state_rcv{ack_done=true, datasize=0}, [], false};
+
+parse({file, make_symlink, _Args, ok},State) ->
+    {State#state_rcv{ack_done=true, datasize=0}, [], false};
+parse({file, make_symlink, [_Existing, New], {error, eexist} },State) ->
+    ?LOGF("error while creating symlink: ~p already exists~n",[New],?NOTICE),
+    {State#state_rcv{ack_done=true, datasize=0}, [], false};
+parse({file, make_symlink, [Existing, New], {error,Reason}},State) ->
+    ?LOGF("error while creating symlink: ~p to ~p (~p)~n",[Existing, New, Reason],?ERR),
+    ts_mon:add({count,error_fs_mksymlink}),
+    {State#state_rcv{ack_done=true, datasize=0}, [], false};
+
 parse({file, delete, [_Path], ok},State) ->
     {State#state_rcv{ack_done=true, datasize=0}, [], false};
 parse({file, delete, [Path], {error,Reason}},State) ->
     ?LOGF("error while deleting file: ~p (~p)~n",[Path, Reason],?ERR),
     {State#state_rcv{ack_done=true, datasize=0}, [], false};
 
+parse({file, read_file_info, [_Path], {ok, _FileInfo}},State) ->
+    %% which value should we use for datasize ?
+    {State#state_rcv{ack_done=true,datasize=0}, [], false};
+parse({file, read_file_info, [Path], {error,Reason}},State) ->
+    ?LOGF("error while running stat file: ~p (~p)~n",[Path,Reason],?ERR),
+    ts_mon:add({count,error_fs_stat}),
+    {State#state_rcv{ack_done=true,datasize=0}, [], false};
+
 parse({ts_utils, read_file_raw, [_Path], {ok,_Res,Size}},State) ->
     {State#state_rcv{ack_done=true,datasize=Size}, [], false};
 parse({ts_utils, read_file_raw, [Path], {error,Reason}},State) ->
@@ -242,6 +265,10 @@ get_message2(#fs{command=del_dir, path=Path}) ->
     {file,del_dir,[Path],0};
 get_message2(#fs{command=make_dir, path=Path}) ->
     {file,make_dir,[Path],0};
+get_message2(#fs{command=make_symlink, path=Existing, dest=New}) ->
+    {file,make_symlink,[Existing, New],0};
+get_message2(#fs{command=stat, path=Path}) ->
+    {file,read_file_info,[Path],0};
 get_message2(#fs{command=write,path=Path, size=Size}) ->
     {file,write_file,[Path,ts_utils:urandomstr(Size),[raw]],Size}.
 
