@@ -19,7 +19,8 @@
 %%%  In addition, as a special exception, you have the permission to
 %%%  link the code of this program with any library released under
 %%%  the EPL license and distribute linked combinations including
-%%%  the two.
+%%%  the two; the MPL (Mozilla Public License), which EPL (Erlang
+%%%  Public License) is based on, is included in this exception.
 
 %%% This module launch clients (ts_client module) given a number of
 %%% clients and the intensity of the arrival process (intensity =
@@ -33,6 +34,7 @@
 -author('nicolas.niclausse@niclux.org').
 
 -include("ts_profile.hrl").
+-include("ts_config.hrl").
 
 % wait up to 10ms after an error
 -define(NEXT_AFTER_FAILED_TIMEOUT, 10).
@@ -114,6 +116,7 @@ wait({launch, [], Seed}, State=#launcher{static_done=Static_done}) ->
     case ts_config_server:get_client_config(MyHostName) of
         {ok, {[{Intensity, Users, Duration}| Rest], StartDate, Max}} ->
             ?LOGF("Expected duration of first phase: ~p sec (~p users) ~n",[Duration/1000, Users], ?NOTICE),
+            check_max_users(Max),
             NewState = State#launcher{phases         = Rest,
                                       nusers         = Users,
                                       phase_nusers   = Users,
@@ -132,17 +135,18 @@ wait({launch, [], Seed}, State=#launcher{static_done=Static_done}) ->
 
 %% start with a already known configuration. This case occurs when a
 %% beam is started by a launcher (maxclients reached)
-wait({launch, {[{Intensity, Users, Duration}| Rest], Max}, Seed}, State) ->
+wait({launch, {[{Intensity, Users, Duration}| Rest], Max, PhaseId}, Seed}, State) ->
     ?LOGF("Starting with ~p users to do in the current phase (max is ~p)~n",
           [Users, Max],?DEB),
     ts_utils:init_seed(Seed),
     ?LOGF("Expected duration of phase: ~p sec ~n",[Duration/1000], ?NOTICE),
     ts_launcher_mgr:check_registered(),
     {next_state, launcher, State#launcher{phases = Rest, nusers = Users,
-                                       phase_nusers = Users,
-                                       phase_duration=Duration,
-                                       phase_start = now(),
-                                       intensity = Intensity, maxusers=Max},
+                                          phase_nusers = Users,
+                                          phase_duration=Duration,
+                                          phase_start = ?NOW,
+                                          phase_id  = PhaseId,
+                                          intensity = Intensity, maxusers=Max},
      State#launcher.short_timeout};
 wait({static,0}, State) ->
     %% static launcher has no work to do, do not wait for him.
@@ -161,7 +165,7 @@ wait_static({static, Static}, State=#launcher{maxusers=Max,intensity=Intensity,
     WarmTimeout = set_warm_timeout(StartDate)+round(ts_stats:exponential(Intensity)),
     Warm = lists:min([WarmTimeout,?config(max_warm_delay)]),
     ?LOGF("Activate launcher (~p users) in ~p msec ~n",[Users, Warm], ?NOTICE),
-    PhaseStart = ts_utils:add_time(now(), Warm div 1000),
+    PhaseStart = ts_utils:add_time(?NOW, Warm div 1000),
     NewMax = case Max > Static of
                true  ->
                      Max-Static;
@@ -181,10 +185,11 @@ launcher(_Event, State=#launcher{nusers = 0, phases = [] }) ->
 launcher(timeout, State=#launcher{nusers        = Users,
                                   phase_nusers  = PhaseUsers,
                                   phases        = Phases,
+                                  phase_id      = Id,
                                   started_users = Started,
                                   intensity     = Intensity}) ->
-    BeforeLaunch = now(),
-    case do_launch({Intensity,State#launcher.myhostname}) of
+    BeforeLaunch = ?NOW,
+    case do_launch({Intensity,State#launcher.myhostname,Id}) of
         {ok, Wait} ->
             case check_max_raised(State) of
                 true ->
@@ -201,25 +206,26 @@ launcher(timeout, State=#launcher{nusers        = Users,
                             ?LOGF("Start a new arrival phase (~p users, ~p); expected duration=~p sec~n",
                                   [NewUsers, NewIntensity, PhaseLength/1000], ?NOTICE),
                             {next_state,launcher,State#launcher{phases = Rest,
-                                                             nusers = NewUsers,
-                                                             phase_nusers = NewUsers,
-                                                             phase_duration=PhaseLength,
-                                                             phase_start = now(),
-                                                             intensity = NewIntensity},
+                                                                nusers = NewUsers,
+                                                                phase_nusers = NewUsers,
+                                                                phase_duration=PhaseLength,
+                                                                phase_start = ?NOW,
+                                                                phase_id  = Id+1,
+                                                                intensity = NewIntensity},
                              round(Wait)};
                         {stop} ->
                             {stop, normal, State};
                         {continue} ->
-                            Now=now(),
+                            Now=?NOW,
                             LaunchDuration = ts_utils:elapsed(BeforeLaunch, Now),
                             %% to keep the rate of new users as expected,
                             %% remove the time to launch a client to the next
                             %% wait.
                             NewWait = case Wait > LaunchDuration of
-                                          true -> round(Wait - LaunchDuration);
+                                          true -> trunc(Wait - LaunchDuration);
                                           false -> 0
                                       end,
-                            ?DebugF("Real Wait =~p ~n", [NewWait]),
+                            ?DebugF("Real Wait = ~p (was ~p)~n", [NewWait,Wait]),
                             {next_state,launcher,State#launcher{nusers = Users-1, started_users=Started+1} , NewWait}
                     end
             end;
@@ -294,19 +300,22 @@ code_change(_OldVsn, StateName, StateData, _Extra) ->
 %%%----------------------------------------------------------------------
 
 %%% @spec skip_empty_phase(record(launcher)) -> {next_state, launcher, record(launcher)}
-%%% @doc if a phase contains no users, sleep, before tryinh the next one @end
-skip_empty_phase(State=#launcher{phases=Phases,phase_duration=Duration})->
+%%% @doc if a phase contains no users, sleep, before trying the next one @end
+skip_empty_phase(State=#launcher{phases=Phases,phase_id=Id,phase_duration=Duration})->
     ?LOGF("No user, skip phase (~p ~p)~n",[Phases,Duration],?INFO),
     case change_phase(0, Phases, 0, {Duration, 0}) of
+        {stop} ->
+            {stop, normal, State};
         {change, 0, _, PhaseLength, Rest} ->
             %% next phase is also empty, loop
-            skip_empty_phase(State#launcher{phases=Rest,phase_duration=PhaseLength});
+            skip_empty_phase(State#launcher{phases=Rest,phase_duration=PhaseLength,phase_id=Id+1});
         {change, NewUsers, NewIntensity, PhaseLength,Rest} ->
             {next_state,launcher,State#launcher{phases = Rest,
                                                 nusers = NewUsers,
                                                 phase_nusers = NewUsers,
                                                 phase_duration=PhaseLength,
-                                                phase_start = now(),
+                                                phase_start = ?NOW,
+                                                phase_id = Id+1,
                                                 intensity = NewIntensity}, 1}
     end.
 
@@ -351,10 +360,10 @@ change_phase(_N, _, _Current, {_Total, _}) ->
 %%%----------------------------------------------------------------------
 %%% Func: check_max_raised/1
 %%%----------------------------------------------------------------------
-check_max_raised(State=#launcher{phases=Phases,maxusers=Max,nusers=Users,
-                              started_users=Started, phase_start=Start, phase_duration=Duration,
-                              intensity=Intensity}) when Started >= Max ->
-    PendingDuration = Duration - ts_utils:elapsed(Start, now()),
+check_max_raised(State=#launcher{phases=Phases,maxusers=Max,nusers=Users, phase_id=Id,
+                                 started_users=Started, phase_start=Start, phase_duration=Duration,
+                                 intensity=Intensity}) when Started >= Max ->
+    PendingDuration = Duration - ts_utils:elapsed(Start, ?NOW),
     ActiveClients =  ts_client_sup:active_clients(),
     ?DebugF("Current active clients on beam: ~p (max is ~p)~n", [ActiveClients, State#launcher.maxusers]),
     case ActiveClients >= Max of
@@ -364,7 +373,7 @@ check_max_raised(State=#launcher{phases=Phases,maxusers=Max,nusers=Users,
                        0 ->  Phases;
                        _ -> [{Intensity,Users-1,PendingDuration}|Phases]
                    end,
-            ts_config_server:newbeam(list_to_atom(State#launcher.myhostname), {Args, Max}),
+            ts_config_server:newbeam(list_to_atom(State#launcher.myhostname), {Args, Max, Id}),
             true;
         false ->
             ?DebugF("Current clients on beam: ~p~n", [ActiveClients]),
@@ -377,10 +386,10 @@ check_max_raised(_State) -> % number of started users less than max, no need to
 %%%----------------------------------------------------------------------
 %%% Func: do_launch/1
 %%%----------------------------------------------------------------------
-do_launch({Intensity, MyHostName})->
+do_launch({Intensity, MyHostName, PhaseId})->
     %%Get one client
     %%set the profile of the client
-    case catch ts_config_server:get_next_session(MyHostName) of
+    case catch ts_config_server:get_next_session({MyHostName, PhaseId} ) of
         [{'EXIT', {timeout, _ }}] ->
             ?LOG("get_next_session failed (timeout), skip this session !~n", ?ERR),
             ts_mon:add({ count, error_next_session }),
@@ -397,7 +406,7 @@ do_launch({Intensity, MyHostName})->
     end.
 
 set_warm_timeout(StartDate)->
-    case ts_utils:elapsed(now(), StartDate) of
+    case ts_utils:elapsed(?NOW, StartDate) of
         WaitBeforeStart when WaitBeforeStart>0 ->
             round(WaitBeforeStart);
         _Neg ->
@@ -406,3 +415,19 @@ set_warm_timeout(StartDate)->
                  "Anyway, start launcher NOW! ~n", ?WARN),
             1
     end.
+
+check_max_users(Max) ->
+    try
+        Data = os:cmd("grep \"open files\"  /proc/self/limits"),
+        {match,[Val]} = re:run(Data,"Max open files\\s+(\\d+)",[{capture,all_but_first,list}]),
+        Limit = list_to_integer(Val),
+        case (Max > Limit ) of
+            true ->
+                ?LOGF("WARNING !!! too few file descriptors available (~w), you should decrease maxusers (currently ~w)",[Limit,Max], ?CRIT);
+            false ->
+                ?LOGF("maxusers is below file descriptors limit (~p)",[Limit], ?DEB)
+        end
+    catch
+        _Error:_Reason ->
+            ?LOG("Can't get file descriptors limit from system, you should verify that 'maxusers' has a good value ",?NOTICE)
+    end.
