@@ -19,7 +19,8 @@
 %%%  In addition, as a special exception, you have the permission to
 %%%  link the code of this program with any library released under
 %%%  the EPL license and distribute linked combinations including
-%%%  the two.
+%%%  the two; the MPL (Mozilla Public License), which EPL (Erlang
+%%%  Public License) is based on, is included in this exception.
 
 -module(ts_plugin).
 
@@ -28,8 +29,7 @@
 -export([behaviour_info/1]).
 
 behaviour_info(callbacks) ->
-    [{init_dynparams,0},
-     {add_dynparams, 4},
+    [{add_dynparams, 4},
      {get_message, 2},
      {session_defaults, 0},
      {dump, 2},
