@@ -16,7 +16,8 @@
 %%%  In addition, as a special exception, you have the permission to
 %%%  link the code of this program with any library released under
 %%%  the EPL license and distribute linked combinations including
-%%%  the two.
+%%%  the two; the MPL (Mozilla Public License), which EPL (Erlang
+%%%  Public License) is based on, is included in this exception.
 
 %%% File    : ts_search.erl
 %%% Author  : Mickael Remond <mickael.remond@erlang-fr.org>
@@ -31,17 +32,22 @@
 -module(ts_search).
 -vc('$Id$ ').
 
--export([subst/2, match/4, parse_dynvar/2]).
+-export([subst/2, match/5, parse_dynvar/2]).
 
+-include("ts_macros.hrl").
 -include("ts_profile.hrl").
 
+%% @type dynvar() = {Key::atom(), Value::string()} | [].
+%% @type dynvars() = [dynvar()]
+
 %% ----------------------------------------------------------------------
-%% Function: subst/2
-%% Purpose: search into a given string and replace %%Mod:Fun%% strings
-%%          by the result of the call to Mod:Fun(Pid) where Pid is the
-%%          Pid of the client The substitution tag are intended to
-%%          be used in tsung.xml scenarii files.
-%% Returns: new string
+%% @spec subst(Data::term(), DynVar::dynvars() ) -> term()
+%% @doc search into a given string and replace %%Mod:Fun%% (resp
+%%          %%__Variable%%) strings by the result of the call to
+%%          Mod:Fun({Pid, DynVars }) (resp the value of the variable) where Pid
+%%          is the Pid of the client. The substitution tag are
+%%          intended to be used in tsung.xml scenarii files.
+%% @end
 %% ----------------------------------------------------------------------
 subst(Int, _DynVar) when is_integer(Int) ->
     Int;
@@ -111,40 +117,40 @@ extract_function([H|Tail],DynVar,  Acc, Mod, Fun) ->
     extract_function(Tail, DynVar, Acc, Mod, [H|Fun]).
 
 %%----------------------------------------------------------------------
-%% @spec match(Match::#match{}, Data::binary() | list, {Counts::integer(),
-%% Max::integer(), SessionId::integer(),UserId::integer()}, Dynvars::term()
-%%  ) ->  Count::integer()
+%% @spec match(Match::#match{}, Data::binary() | list,
+%%             {Counts::integer(), Max::integer(), SessionId::integer(), UserId::integer()},
+%%             Dynvars::term(), Transactions::list() ) -> Count::integer()
 %% @doc search for regexp in Data; send result to ts_mon
 %% @end
 %%----------------------------------------------------------------------
-match([], _Data, {Count, _MaxC, _SessionId, _UserId}, _DynVars) -> Count;
-match([Match=#match{'skip_headers'=http}|Tail], Data, Counts,DynVars) when is_binary(Data)->
+match([], _Data, {Count, _MaxC, _SessionId, _UserId}, _DynVars, _Tr) -> Count;
+match([Match=#match{'skip_headers'=http}|Tail], Data, Counts, DynVars, Tr) when is_binary(Data)->
     %% keep http body only
     case re:run(Data,"\\r\\n\\r\\n(.*)",[{capture,all_but_first,binary},dotall]) of
         {match,[NewData]} ->
-            match([Match#match{'skip_headers'=no}|Tail], NewData, Counts, DynVars);
+            match([Match#match{'skip_headers'=no}|Tail], NewData, Counts, DynVars, Tr);
         _ ->
             ?LOGF("Skip http headers failure, data was: ~p ~n",[Data], ?ERR),
-            match([Match#match{'skip_headers'=no}|Tail], Data, Counts, DynVars)
+            match([Match#match{'skip_headers'=no}|Tail], Data, Counts, DynVars, Tr)
         end;
 
-match([Match=#match{'apply_to_content'=undefined}|Tail], Data, Counts,DynVars) ->
+match([Match=#match{'apply_to_content'=undefined}|Tail], Data, Counts,DynVars,Tr) ->
     ?DebugF("Matching Data size ~p; apply undefined~n",[size(Data)]),
-    match([Match|Tail], Data, Counts, [],DynVars);
-match([Match=#match{'apply_to_content'={Module,Fun}}|Tail], Data, Counts,DynVars) ->
+    match([Match|Tail], Data, Counts, [],DynVars, Tr);
+match([Match=#match{'apply_to_content'={Module,Fun}}|Tail], Data, Counts,DynVars,Tr) ->
     ?DebugF("Matching Data size ~p; apply ~p:~p~n",[size(Data),Module,Fun]),
     NewData = Module:Fun(Data),
     ?DebugF("Match: apply result =~p~n",[NewData]),
-    match([Match|Tail], NewData, Counts, [],DynVars).
+    match([Match|Tail], NewData, Counts, [],DynVars, Tr).
 
 %% @spec match(Match::#match{}, Data::binary() | list(), Count::tuple(),
-%%             Stats::list(), DynVars::term()) -> Count::integer()
-match([], _Data, {Count,_, _,_}, Stats, _) ->
+%%             Stats::list(), DynVars::term(), Transaction::atom()) -> Count::integer()
+match([], _Data, {Count,_, _,_}, Stats, _, _) ->
     %% all matches done, add stats, and return Count unchanged (continue)
     ts_mon:add(Stats),
     Count;
 match([Match=#match{regexp=RawRegExp,subst=Subst, do=Action, 'when'=When}
-       |Tail], Data,Counts,Stats,DynVars)->
+       |Tail], Data,Counts,Stats,DynVars, Tr)->
     RegExp  = case Subst of
         true -> subst(RawRegExp, DynVars);
         _    -> RawRegExp
@@ -153,10 +159,20 @@ match([Match=#match{regexp=RawRegExp,subst=Subst, do=Action, 'when'=When}
     case re:run(Data, RegExp) of
         {When,_} ->
             ?LOGF("Ok Match (regexp=~p) do=~p~n",[RegExp,Action], ?INFO),
-            setcount(Match, Counts, [{count, match}| Stats], Data);
+            case Action of
+                Act when Act =:= 'continue'; Act =:= 'log'; Act =:= 'dump' ->
+                    setcount(Match, Counts, [{count, match}| Stats], Data, Tr),
+                    match(Tail, Data, Counts, Stats,DynVars, Tr);
+                _       -> setcount(Match, Counts, [{count, match}| Stats], Data, Tr)
+            end;
         When -> % nomatch
             ?LOGF("Bad Match (regexp=~p) do=~p~n",[RegExp, Action], ?INFO),
-            setcount(Match, Counts, [{count, nomatch} | Stats],Data);
+            case Action of
+                Act when Act =:= 'continue'; Act =:= 'log'; Act =:= 'dump' ->
+                    setcount(Match, Counts, [{count, nomatch}| Stats], Data, Tr),
+                    match(Tail, Data, Counts, Stats,DynVars, Tr);
+                _       -> setcount(Match, Counts, [{count, nomatch}| Stats], Data, Tr)
+            end;
         {match,_} -> % match but when=nomatch
             ?LOGF("Ok Match (regexp=~p)~n",[RegExp], ?INFO),
             case Action of
@@ -164,7 +180,7 @@ match([Match=#match{regexp=RawRegExp,subst=Subst, do=Action, 'when'=When}
                 restart -> put(restart_count, 0);
                 _       -> ok
             end,
-            match(Tail, Data, Counts, [{count, match} | Stats],DynVars);
+            match(Tail, Data, Counts, [{count, match} | Stats],DynVars, Tr);
         nomatch -> % nomatch but when=match
             ?LOGF("Bad Match (regexp=~p)~n",[RegExp], ?INFO),
             case Action of
@@ -172,10 +188,7 @@ match([Match=#match{regexp=RawRegExp,subst=Subst, do=Action, 'when'=When}
                 restart -> put(restart_count, 0);
                 _       -> ok
             end,
-            match(Tail, Data, Counts,[{count, nomatch} | Stats],DynVars);
-        {error,_Error} ->
-            ?LOGF("Error while matching: bad REGEXP (~p)~n", [RegExp], ?ERR),
-            match(Tail, Data, Counts,[{count, badregexp} | Stats],DynVars)
+            match(Tail, Data, Counts,[{count, nomatch} | Stats],DynVars,Tr)
     end.
 
 %%----------------------------------------------------------------------
@@ -186,25 +199,25 @@ match([Match=#match{regexp=RawRegExp,subst=Subst, do=Action, 'when'=When}
 %%   - if restart is true, we must start again the whole session, set count to MaxCount
 %%   - if stop is true, set count to 0
 %%----------------------------------------------------------------------
-setcount(#match{do=continue}, {Count, _MaxC, _SessionId, _UserId}, Stats,_)->
+setcount(#match{do=continue}, {Count, _MaxC, _SessionId, _UserId}, Stats,_,_)->
     ts_mon:add(Stats),
     Count;
-setcount(#match{do=log}, {Count, MaxC, SessionId, UserId}, Stats,_)->
-    ts_mon:add_match(Stats,{UserId,SessionId,MaxC-Count}),
+setcount(#match{do=log, name=Name}, {Count, MaxC, SessionId, UserId}, Stats,_,Tr)->
+    ts_mon:add_match(Stats,{UserId,SessionId,MaxC-Count,Tr, Name}),
     Count;
-setcount(#match{do=dump}, {Count, MaxC, SessionId, UserId}, Stats,Data)->
-    ts_mon:add_match(Stats,{UserId,SessionId,MaxC-Count, Data}),
+setcount(#match{do=dump, name=Name}, {Count, MaxC, SessionId, UserId}, Stats, Data, Tr)->
+    ts_mon:add_match(Stats,{UserId,SessionId,MaxC-Count, Data, Tr, Name}),
     Count;
-setcount(#match{do=restart, max_restart=MaxRestart}, {Count, MaxC,SessionId,UserId}, Stats,_)->
+setcount(#match{do=restart, max_restart=MaxRestart, name=Name}, {Count, MaxC,SessionId,UserId}, Stats,_, Tr)->
     CurRestart = get(restart_count),
-    Ids={UserId,SessionId,MaxC-Count},
+    Ids={UserId,SessionId,MaxC-Count,Tr,Name},
     ?LOGF("Restart on (no)match ~p~n",[CurRestart], ?INFO),
     case CurRestart of
         undefined ->
             put(restart_count,1),
             ts_mon:add_match([{count, match_restart} | Stats],Ids),
             MaxC ;
-        Val when Val > MaxRestart ->
+        Val when Val >= MaxRestart ->
             ?LOG("Max restart reached, abort ! ~n", ?WARN),
             ts_mon:add_match([{count, match_restart_abort} | Stats],Ids),
             0;
@@ -213,7 +226,7 @@ setcount(#match{do=restart, max_restart=MaxRestart}, {Count, MaxC,SessionId,User
             ts_mon:add_match([{count, match_restart} | Stats],Ids),
             MaxC
     end;
-setcount(#match{do=loop,loop_back=Back,max_loop=MaxLoop,sleep_loop=Sleep},{Count,_MaxC,_SessionId,_UserId},Stats,_)->
+setcount(#match{do=loop,loop_back=Back,max_loop=MaxLoop,sleep_loop=Sleep},{Count,_MaxC,_SessionId,_UserId},Stats,_,_)->
     CurLoop = get(loop_count),
     ?LOGF("Loop on (no)match ~p~n",[CurLoop], ?INFO),
     ts_mon:add([{count, match_loop} | Stats]),
@@ -231,15 +244,14 @@ setcount(#match{do=loop,loop_back=Back,max_loop=MaxLoop,sleep_loop=Sleep},{Count
             timer:sleep(Sleep),
             Count + 1 + Back
     end;
-setcount(#match{do=abort}, {Count,MaxC,SessionId,UserId}, Stats,_) ->
-    ts_mon:add_match([{count, match_stop} | Stats],{UserId,SessionId,MaxC-Count}),
+setcount(#match{do=abort,name=Name}, {Count,MaxC,SessionId,UserId}, Stats,_, Tr) ->
+    ts_mon:add_match([{count, match_stop} | Stats],{UserId,SessionId,MaxC-Count,Tr, Name}),
     0.
 
 %%----------------------------------------------------------------------
-%% Func: parse_dynvar/2
-%% Args: DynVarSpecs, Data
-%% Purpose: look for dynamic variables in Data
-%% Returns: DynVars (List)
+%% @spec parse_dynvar(Dynvarspecs::list(), Data::binary | list) -> dynvars()
+%% @doc Look for dynamic variables in Data
+%% @end
 %%----------------------------------------------------------------------
 parse_dynvar([], _Data) -> ts_dynvars:new();
 parse_dynvar(DynVarSpecs, Data)  when is_binary(Data) ->
@@ -260,20 +272,69 @@ parse_dynvar(DynVarSpecs, _Data)  ->
 %            regexp or xpath variables respectively
 parse_dynvar([],_Binary , _String,_Tree, DynVars) -> DynVars;
 
-
-
-parse_dynvar(D=[{re,_, _}| _],Binary,undefined,Tree,DynVars) ->
+parse_dynvar(D=[{re,_, _, _}| _],Binary,undefined,Tree,DynVars) ->
     parse_dynvar(D,Binary,Binary,Tree,DynVars);
-parse_dynvar([{re,VarName, RegExp}| DynVarsSpecs],Binary,Data,Tree,DynVars) ->
+parse_dynvar([{re,Name,RE}| Tail],Binary,Data,Tree,DynVars) ->
+    parse_dynvar([{re,Name, RE, undefined}| Tail],Binary,Data,Tree,DynVars);
+parse_dynvar([{re,VarName, RegExp, Apply}| DynVarsSpecs],Binary,Data,Tree,DynVars) ->
     case re:run(Data, RegExp,[{capture,[1],binary}]) of
         {match,[Value]} ->
-            ?LOGF("DynVar (RE): Match (~p=~p) ~n",[VarName, Value], ?INFO),
-            parse_dynvar(DynVarsSpecs, Binary,Data,Tree,
-                            ts_dynvars:set(VarName,Value,DynVars));
+            ConvValue = apply_fun(Apply,Value),
+            ?LOGF("DynVar (RE): Match (~p=~p) Converted: ~p~n",[VarName, Value, ConvValue], ?INFO),
+            parse_dynvar(DynVarsSpecs,Binary,Data,Tree, ts_dynvars:set(VarName,ConvValue,DynVars));
         nomatch ->
             ?LOGF("Dyn Var (RE): no Match (varname=~p), ~n",[VarName], ?WARN),
             ?LOGF("Regexp was: ~p ~n",[RegExp], ?INFO),
-            parse_dynvar(DynVarsSpecs, Binary,Data,Tree, ts_dynvars:set(VarName,"",DynVars))
+            parse_dynvar(DynVarsSpecs,Binary,Data,Tree, ts_dynvars:set(VarName,<< >> ,DynVars))
+    end;
+
+parse_dynvar([{header,VarName, HeaderName}| DynVarsSpecs],
+	     Binary,String,Tree, DynVars) ->
+    BinHeaders = extract_headers(Binary),
+    Headers = mochiweb_headers:from_binary(BinHeaders),
+    case string:tokens(HeaderName, "/") of
+      [H1] ->
+        V1 = mochiweb_headers:get_value(H1, Headers),
+        ?LOGF("DynVar: Header (~p=~p) ~n",[VarName, V1], ?NOTICE),
+        parse_dynvar(DynVarsSpecs, Binary,String,Tree,
+                        ts_dynvars:set(VarName,V1,DynVars));
+      [H1,SubH] ->
+        Value = case mochiweb_headers:get_value(H1, Headers) of 
+          [] ->
+            {ok, Old} = ts_dynvars:lookup(VarName, DynVars, ""),
+            ?LOGF("DynVar: Header ~p not found ; using ~p ~n",[H1, Old], ?WARN),
+            Old;
+          undefined ->
+            {ok, Old} = ts_dynvars:lookup(VarName, DynVars, ""),
+            ?LOGF("DynVar: Header ~p not found ; using ~p ~n",[H1, Old], ?WARN),
+            Old;
+          SubV when H1 == "www-authenticate" orelse H1 == "authentication-info"->
+            ?LOGF("DynVar: Found header ~p ~n",[SubV], ?WARN),
+            {_, Params} = parse_header(SubV, ","),
+            ?LOGF("DynVar: Parsed subheader ~p ~n",[Params], ?WARN),
+            case lists:keyfind(SubH, 1, Params) of
+              false ->
+                {ok, Old} = ts_dynvars:lookup(VarName, DynVars, ""),
+                ?LOGF("DynVar: SubHeader ~p not found ; using ~p ~n",[VarName, Old], ?WARN),
+                Old;
+              {_, V} ->
+                ?LOGF("DynVar: SubHeader (~p=~p) ~n",[VarName, V], ?DEB),
+                V
+            end; 
+          SubV ->
+            {_, Params}= parse_header(SubV, ";"),
+            case lists:keyfind(SubH, 1, Params) of
+              false ->
+                ?LOGF("DynVar: SubHeader ~p not found ~n",[VarName], ?WARN),
+                {ok, Old} = ts_dynvars:lookup(VarName, DynVars, ""),
+                Old;
+              {_, V} ->
+                ?LOGF("DynVar: SubHeader (~p=~p) ~n",[VarName, V], ?NOTICE),
+                V
+            end
+        end,
+        parse_dynvar(DynVarsSpecs, Binary,String,Tree,
+                        ts_dynvars:set(VarName,Value,DynVars))
     end;
 
 parse_dynvar(D=[{xpath,_VarName, _Expr}| _DynVarsSpecs],
@@ -295,6 +356,8 @@ parse_dynvar(D=[{xpath,_VarName, _Expr}| _DynVarsSpecs],
             parse_dynvar(D,Binary,String,xpath_error,DynVars)
     end;
 
+
+
 parse_dynvar(D=[{jsonpath,_VarName, _Expr}| _DynVarsSpecs],
                 Binary,String,undefined,DynVars) ->
     Body = extract_body(Binary),
@@ -314,6 +377,7 @@ parse_dynvar(D=[{pgsql_expr,_VarName, _Expr}| _DynVarsSpecs],
     Pairs=ts_pgsql:to_pairs(Binary),
     parse_dynvar(D,Binary,String,Pairs,DynVars);
 
+
 parse_dynvar([{xpath,VarName,_Expr}|DynVarsSpecs],Binary,String,xpath_error,DynVars)->
     ?LOGF("Couldn't execute XPath: page not parsed (varname=~p)~n",
           [VarName],?ERR),
@@ -329,33 +393,49 @@ parse_dynvar([{pgsql_expr,VarName,_Expr}|DynVarsSpecs],Binary,String,pgsql_error
     parse_dynvar(DynVarsSpecs, Binary,String,json_error,DynVars);
 
 parse_dynvar([{xpath,VarName, Expr}| DynVarsSpecs],Binary,String,Tree,DynVars)->
-    Value = mochiweb_xpath:execute(Expr,Tree),
-    ?DebugF("Xpath result: ~p~n", [Value]),
-    case Value of
-        [] -> ?LOGF("Dyn Var: no Match (varname=~p), ~n",[VarName],?WARN);
-        _  -> ?LOGF("Dyn Var: Match (~p=~p), ~n",[VarName,Value],?INFO)
-    end,
+    Value = case mochiweb_xpath:execute(Expr,Tree) of
+                [] ->
+                    ?LOGF("Dyn Var: no Match (varname=~p), ~n",[VarName],?WARN),
+                    << >>;
+                Val  ->
+                    ?LOGF("Dyn Var: Match (~p=~p), ~n",[VarName,Val],?INFO),
+                    Val
+            end,
     parse_dynvar(DynVarsSpecs, Binary,String,Tree,ts_dynvars:set(VarName,Value,DynVars));
 
 parse_dynvar([{jsonpath,VarName, Expr}| DynVarsSpecs],Binary,String,JSON,DynVars)->
-    Values = ts_utils:jsonpath(Expr,JSON),
-    case Values of
-        undefined -> ?LOGF("Dyn Var: no Match (varname=~p), ~n",[VarName],?WARN);
-        _  -> ?LOGF("Dyn Var: Match (~p=~p), ~n",[VarName,Values],?INFO)
-    end,
+    Values = case ts_utils:jsonpath(Expr,JSON) of
+                undefined ->
+                    ?LOGF("Dyn Var: no Match (varname=~p), ~n",[VarName],?WARN),
+                     << >>;
+                 {struct, Struct}  ->
+                     ?LOGF("Dyn Var: Match (~p=~p), ~n",[VarName,Struct],?INFO),
+                     iolist_to_binary(mochijson2:encode({struct, Struct}));
+                 Val  ->
+                     ?LOGF("Dyn Var: Match (~p=~p), ~n",[VarName,Val],?INFO),
+                     Val
+             end,
     parse_dynvar(DynVarsSpecs, Binary,String,JSON,ts_dynvars:set(VarName,Values,DynVars));
 
 parse_dynvar([{pgsql_expr,VarName, Expr}| DynVarsSpecs],Binary,String,PGSQL,DynVars)->
-    Values = ts_pgsql:find_pair(Expr,PGSQL),
-    case Values of
-        undefined -> ?LOGF("Dyn Var: no Match (varname=~p), ~n",[VarName],?WARN);
-        _  -> ?LOGF("Dyn Var: Match (~p=~p), ~n",[VarName,Values],?INFO)
-    end,
+    Values = case ts_pgsql:find_pair(Expr,PGSQL) of
+                 undefined ->
+                     ?LOGF("Dyn Var: no Match (varname=~p), ~n",[VarName],?WARN),
+                     << >>;
+                 Val  ->
+                     ?LOGF("Dyn Var: Match (~p=~p), ~n",[VarName,Val],?INFO),
+                     Val
+                 end,
     parse_dynvar(DynVarsSpecs, Binary,String,PGSQL,ts_dynvars:set(VarName,Values,DynVars));
 
 parse_dynvar(Args, _Binary,_String,_Tree, _DynVars) ->
     ?LOGF("Bad args while parsing Dyn Var (~p)~n", [Args], ?ERR),
-    [].
+    << >>.
+
+apply_fun(undefined, Value) ->
+    Value;
+apply_fun(Fun,Value) ->
+    Fun(Value).
 
 extract_body(Data) ->
     case re:run(Data,"\r\n\r\n(.*)$",[{capture,all_but_first,binary},dotall]) of
@@ -363,3 +443,45 @@ extract_body(Data) ->
         {match, [Val]} -> Val;
         _              -> Data
     end.
+    
+extract_headers(<<"\r\n",Rest/binary>>) ->
+    Rest;
+extract_headers(<<_:1/binary,Rest/binary>>) ->
+    extract_headers(Rest);
+extract_headers(<<>>) ->
+    <<>>.
+    
+%% Comes from mochiweb_utils.erl ; very slightly adapted.
+parse_header(String, ";")-> % for Content-Type and friends
+    [Type | Parts] = [string:strip(S) || S <- string:tokens(String, ";")],
+    {string:to_lower(Type),
+     lists:foldr(fun prepare_headers/2, [], Parts)};
+parse_header(String, ",")-> % for Auth
+    [Type | Rest] = [string:strip(S) || S <- string:tokens(String, " ")],
+    Parts = [string:strip(S) || S <- string:tokens(string:join(Rest, " "), ",")],
+    {string:to_lower(Type),
+     lists:foldr(fun prepare_headers/2, [], Parts)}.
+unquote_header("\"" ++ Rest) ->
+    unquote_header(Rest, []);
+unquote_header(S) ->
+    S.
+prepare_headers(S, Acc)->
+    case lists:splitwith(fun (C) -> C =/= $= end, S) of
+        {"", _} ->
+            %% Skip anything with no name
+            Acc;
+        {_, ""} ->
+            %% Skip anything with no value
+            Acc;
+        {Name, [$\= | Value]} ->
+            [{string:to_lower(string:strip(Name)),
+              unquote_header(string:strip(Value))} | Acc]
+    end.
+unquote_header("", Acc) ->
+    lists:reverse(Acc);
+unquote_header("\"", Acc) ->
+    lists:reverse(Acc);
+unquote_header([$\\, C | Rest], Acc) ->
+    unquote_header(Rest, [C | Acc]);
+unquote_header([C | Rest], Acc) ->
+    unquote_header(Rest, [C | Acc]).
