@@ -1,5 +1,5 @@
 %%%
-%%%  Copyright  IDEALX S.A.S. 2003
+%%%  Copyright © IDEALX S.A.S. 2003
 %%%
 %%%  Author : Nicolas Niclausse <nicolas.niclausse@niclux.org>
 %%%  Created: 04 Dec 2003 by Nicolas Niclausse <nicolas.niclausse@niclux.org>
@@ -21,7 +21,8 @@
 %%%  In addition, as a special exception, you have the permission to
 %%%  link the code of this program with any library released under
 %%%  the EPL license and distribute linked combinations including
-%%%  the two.
+%%%  the two; the MPL (Mozilla Public License), which EPL (Erlang
+%%%  Public License) is based on, is included in this exception.
 
 %%%-------------------------------------------------------------------
 %%% File    : ts_config_server.erl
@@ -51,10 +52,10 @@
          get_client_config/1, newbeams/1, newbeam/2,
          get_monitor_hosts/0, encode_filename/1, decode_filename/1,
          endlaunching/1, status/0, start_file_server/1, get_user_agents/0,
-         get_client_config/2, get_user_param/1, get_jobs_state/0 ]).
+         get_client_config/2, get_user_param/1, get_user_port/1, get_jobs_state/0 ]).
 
 %%debug
--export([choose_client_ip/1, choose_session/1]).
+-export([choose_client_ip/1, choose_session/3]).
 
 %% gen_server callbacks
 -export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2,
@@ -75,6 +76,8 @@
                 total_weight      % total weight of client machines
                }).
 
+-define(RPC_TIMEOUT, 30000).
+
 %%====================================================================
 %% External functions
 %%====================================================================
@@ -94,13 +97,14 @@ status() ->
 %%--------------------------------------------------------------------
 newbeams(HostList)->
     gen_server:cast({global, ?MODULE},{newbeams, HostList }).
+
 %%--------------------------------------------------------------------
 %% Function: newbeam/2
 %% Description: start a new beam with given config. Use by launcher
 %%  when maxclient is reached. In this case, the arrival rate is known
 %%--------------------------------------------------------------------
-newbeam(Host, {Arrivals, MaxUsers})->
-    gen_server:cast({global, ?MODULE},{newbeam, Host, {Arrivals, MaxUsers} }).
+newbeam(Host, Args)->
+    gen_server:cast({global, ?MODULE},{newbeam, Host, Args }).
 
 %%--------------------------------------------------------------------
 %% Function: get_req/2
@@ -140,24 +144,28 @@ get_client_config(Type, Host)->
     gen_server:call({global,?MODULE},{get_client_config, Type, Host}, ?config_timeout).
 
 %%--------------------------------------------------------------------
-%% Function: get_monitor_hosts/0
-%% Returns: [Hosts]
+%% @spec get_monitor_hosts() -> List
+%%  List = [Hosts::string()]
+%% @doc get list of hosts to monitor  @end
 %%--------------------------------------------------------------------
 get_monitor_hosts()->
         gen_server:call({global,?MODULE},{get_monitor_hosts}).
 
 %%--------------------------------------------------------------------
-%% @spec get_next_session(Host::string())-> {ok, SessionId::integer(),
-%%              SessionSize::integer(),IP::tuple(), UserId::integer()}
+%% @spec get_next_session({Host::string(), PhaseId::integer()}) ->
+%%  {ok, SessionId::integer(), SessionSize::integer(), IP::tuple(), UserId::integer() }
 %% @doc Choose randomly a session
 %% @end
 %%--------------------------------------------------------------------
-get_next_session(Host)->
-    gen_server:call({global, ?MODULE},{get_next_session, Host}).
+get_next_session({Host, PhaseId})->
+    gen_server:call({global, ?MODULE},{get_next_session, Host, PhaseId}).
 
 get_user_param(Host)->
     gen_server:call({global, ?MODULE},{get_user_param, Host}).
 
+get_user_port(Ip) ->
+    gen_server:call({global, ?MODULE},{get_user_port, Ip}).
+
 endlaunching(Node) ->
     gen_server:cast({global, ?MODULE},{end_launching, Node}).
 
@@ -197,32 +205,33 @@ init([LogDir]) ->
 handle_call({read_config, ConfigFile}, _From, State=#state{logdir=LogDir}) ->
     case catch ts_config:read(ConfigFile, LogDir) of
         {ok, Config=#config{curid=LastReqId,sessions=[LastSess| Sessions]}} ->
-            case check_config(Config) of
+            ts_utils:init_seed(Config#config.seed),
+            ts_user_server:init_seed(Config#config.seed),
+            application:set_env(tsung_controller, clients, Config#config.clients),
+            application:set_env(tsung_controller, dump, Config#config.dump),
+            application:set_env(tsung_controller, stats_backend, Config#config.stats_backend),
+            application:set_env(tsung_controller, debug_level, Config#config.loglevel),
+            SumWeights = fun(X, Sum) -> X#client.weight + Sum end,
+            Sum = lists:foldl(SumWeights, 0, Config#config.clients),
+            %% we only know now the size of last session from the file: add it
+            %% in the table
+            print_info(),
+            NewLast=LastSess#session{size = LastReqId, type=Config#config.main_sess_type},
+            %% start the file server (if defined) using a separate process (it can be long)
+            spawn(?MODULE, start_file_server, [Config]),
+            ConfigTmp = loop_load(sort_static(Config#config{sessions=[NewLast]++Sessions})),
+            %% Compute per phase popularities
+            NewConfig = compute_popularities(ConfigTmp),
+            ts_job_notify:listen(NewConfig#config.job_notify_port),
+            case check_config(NewConfig) of
                 ok ->
-                    ts_utils:init_seed(Config#config.seed),
-                    ts_user_server:init_seed(Config#config.seed),
-                    application:set_env(tsung_controller, clients, Config#config.clients),
-                    application:set_env(tsung_controller, dump, Config#config.dump),
-                    application:set_env(tsung_controller, stats_backend, Config#config.stats_backend),
-                    application:set_env(tsung_controller, debug_level, Config#config.loglevel),
-                    SumWeights = fun(X, Sum) -> X#client.weight + Sum end,
-                    Sum = lists:foldl(SumWeights, 0, Config#config.clients),
-                    %% we only know now the size of last session from the file: add it
-                    %% in the table
-                    print_info(),
-                    NewLast=LastSess#session{size = LastReqId, type=Config#config.main_sess_type},
-                    %% start the file server (if defined) using a separate process (it can be long)
-                    spawn(?MODULE, start_file_server, [Config]),
-                    NewConfig=loop_load(sort_static(Config#config{sessions=[NewLast]++Sessions})),
-                    set_max_duration(Config#config.duration),
-                    ts_job_notify:listen(Config#config.job_notify_port),
                     {reply, ok, State#state{config=NewConfig, static_users=NewConfig#config.static_users,total_weight = Sum}};
                 {error, Reason} ->
                     ?LOGF("Error while checking config: ~p~n",[Reason],?EMERG),
                     {reply, {error, Reason}, State}
             end;
         {error, {{case_clause, {error, enoent}},
-                  [{xmerl_scan, fetch_DTD, 2}|_]}} ->
+                  [{xmerl_scan, fetch_DTD, 2,_}|_]}} ->
             ?LOG("Error while parsing XML: DTD not found !~n",?EMERG),
             {reply, {error, dtd_not_found}, State};
         {error, Reason} ->
@@ -260,21 +269,28 @@ handle_call({get_user_param, HostName}, _From, State=#state{users=UserId}) ->
     Config = State#state.config,
     {value, Client} = lists:keysearch(HostName, #client.host, Config#config.clients),
     {IPParam, Server} = get_user_param(Client,Config),
-    ts_mon:newclient({static,now()}),
-    {reply, {ok, { IPParam, Server, UserId,Config#config.dump}}, State#state{users=UserId+1}};
+    ts_mon:newclient({static,?NOW}),
+    {reply, {ok, { IPParam, Server, UserId,Config#config.dump,Config#config.seed}}, State#state{users=UserId+1}};
+
+%% get  user port. This is needed by bosh, as there are more than one socket per bosh connection.
+handle_call({get_user_port, IP}, _From, State=#state{ports=Ports}) ->
+    Config = State#state.config,
+    {NewPorts,CPort}   = choose_port(IP, Ports,Config#config.ports_range),
+    {reply, {ok, CPort}, State#state{ports = NewPorts}};
 
 %% get a new session id and user parameters for the given node
-handle_call({get_next_session, HostName}, _From, State=#state{users=Users}) ->
+handle_call({get_next_session, HostName, PhaseId}, _From, State=#state{users=Users}) ->
     Config = State#state.config,
     {value, Client} = lists:keysearch(HostName, #client.host, Config#config.clients),
     ?DebugF("get new session for ~p~n",[_From]),
-    case choose_session(Config#config.sessions) of
+    case choose_session(Config#config.sessions, Config#config.total_popularity, PhaseId) of
         {ok, Session=#session{id=Id}} ->
             ?LOGF("Session ~p choosen~n",[Id],?INFO),
-            ts_mon:newclient({Id,now()}),
+            ts_mon:newclient({Id,?NOW}),
             {IPParam, Server} = get_user_param(Client,Config),
-            {reply, {ok, {Session, IPParam, Server, Users, Config#config.dump}},
-             State#state{users=Users+1}};
+            {reply, {ok, Session#session{client_ip= IPParam, server=Server,userid=Users,
+                                         dump=Config#config.dump, seed=Config#config.seed}},
+             State#state{users=Users+1} };
         Other ->
             {reply, {error, Other}, State}
     end;
@@ -305,7 +321,7 @@ handle_call({get_client_config, Host}, _From, State=#state{curcfg=OldCfg,total_w
     %% set start date if not done yet
     StartDate = set_start_date(State#state.start_date),
     {value, Client} = lists:keysearch(Host, #client.host, Clients),
-    IsLast = OldCfg + 1 >= length(Clients),% test is this is the last launcher to ask for it's config
+    IsLast = OldCfg + 1 >= length(Clients),% test if this is the last launcher to ask for it's config
     Get = fun(Phase,Args)-> {get_client_cfg(Phase,Args),Args} end,
     {Res, _Acc} = lists:mapfoldl(Get, {Total_Weight,Client,IsLast},Config#config.arrivalphases),
     {NewPhases,ClientParams} = lists:unzip(Res),
@@ -326,7 +342,7 @@ handle_call({status}, _From, State) ->
 
 handle_call({get_jobs_state}, _From, State) when State#state.config == undefined ->
     {reply, not_configured, State};
-handle_call({get_jobs_state}, {Pid,Tag}, State) ->
+handle_call({get_jobs_state}, {Pid,_Tag}, State) ->
     Config = State#state.config,
     Reply = case Config#config.job_notify_port of
                 {Ets,Port} ->
@@ -359,34 +375,47 @@ handle_cast({newbeams, HostList}, State=#state{logdir   = LogDir,
             ts_mon:abort(),
             {stop, normal,State};
         Id0 ->
-            Seed=Config#config.seed,
+            Seed = Config#config.seed,
             Args = set_remote_args(LogDir, Config#config.ports_range),
             {BeamsIds, LastId} = lists:mapfoldl(fun(A,Acc) -> {{A, Acc}, Acc+1} end, Id0, RemoteBeams),
             Fun = fun({Host,Id}) -> remote_launcher(Host, Id, Args) end,
             RemoteNodes = ts_utils:pmap(Fun, BeamsIds),
-            ?LOG("All remote beams started, sync ~n",?NOTICE),
+            ?LOG("All remote beams started, syncing ~n",?NOTICE),
             global:sync(),
-            StartLaunchers = fun(Node) ->
-                                     ts_launcher_static:launch({Node,[]}),
-                                     ts_launcher:launch({Node, [], Seed})
-                             end,
-            case Config#config.ports_range of
-                undefined ->
-                    ok;
-                _ ->
-                    ?LOG("Start client port server on remote nodes ~n",?NOTICE),
-                    %% first, get a single erlang node per host, and start the cport gen_server on this node
-                    UNodes = get_one_node_per_host(RemoteNodes),
-                    SetParams = fun(Node) ->
-                            {ok, MyHostName} =ts_utils:node_to_hostname(Node),
-                            {Node, "cport-" ++ MyHostName}
-                          end,
-                    CPorts = lists:map(SetParams, UNodes),
-                    ?LOGF("Will run start_cport with arg:~p ~n",[CPorts],?DEB),
-                    lists:foreach(fun ts_sup:start_cport/1 ,CPorts)
-            end,
-            lists:foreach(StartLaunchers, RemoteNodes),
-            {noreply, State#state{last_beam_id = LastId}}
+            ?LOG("Syncing done, start remote tsung application ~n", ?DEB),
+            {Resl, BadNodes} = rpc:multicall(RemoteNodes,tsung,start,[],?RPC_TIMEOUT),
+            ?LOGF("RPC result: ~p ~p ~n",[Resl,BadNodes],?DEB),
+            case BadNodes of
+                [] ->
+                    StartLaunchers = fun(Node) ->
+                                             ts_launcher_static:launch({Node,[]}),
+                                             ts_launcher:launch({Node, [], Seed})
+                                     end,
+                    case Config#config.ports_range of
+                        undefined ->
+                            ?LOG("Undefined ports_range config ~n",?NOTICE),
+                            ok;
+                        _ ->
+                            ?LOG("Start client port server on remote nodes ~n",?NOTICE),
+                            %% first, get a single erlang node per host, and start the cport gen_server on this node
+                            UNodes = get_one_node_per_host(RemoteNodes),
+                            SetParams = fun(Node) ->
+                                                {ok, MyHostName} =ts_utils:node_to_hostname(Node),
+                                                {Node, "cport-" ++ MyHostName}
+                                        end,
+                            CPorts = lists:map(SetParams, UNodes),
+                            ?LOGF("Will run start_cport with arg:~p ~n",[CPorts],?DEB),
+                            lists:foreach(fun ts_sup:start_cport/1 ,CPorts)
+                    end,
+                    lists:foreach(StartLaunchers, RemoteNodes),
+                    set_max_duration(Config#config.duration),
+
+                    {noreply, State#state{last_beam_id = LastId}};
+                Bad ->
+                    ?LOGF("Can't start tsung application on all remote clients, abort ~p~n",[Bad],?ERR),
+                    ts_mon:abort(),
+                    {stop,normal,State}
+            end
     end;
 
 
@@ -399,12 +428,12 @@ handle_cast({newbeam, Host, _}, State=#state{ hostname=LocalHost,config=Config})
     {noreply, State};
 
 %% start a launcher on a new beam with slave module
-handle_cast({newbeam, Host, Arrivals}, State=#state{last_beam_id = NodeId, config=Config, logdir = LogDir}) ->
+handle_cast({newbeam, Host, Args}, State=#state{last_beam_id = NodeId, config=Config, logdir = LogDir}) ->
     Args = set_remote_args(LogDir,Config#config.ports_range),
     Seed = Config#config.seed,
     Node = remote_launcher(Host, NodeId, Args),
     ts_launcher_static:stop(Node), % no need for static launcher in this case (already have one)
-    ts_launcher:launch({Node, Arrivals, Seed}),
+    ts_launcher:launch({Node, Args, Seed}),
     {noreply, State#state{last_beam_id = NodeId+1}};
 
 handle_cast({end_launching, _Node}, State=#state{ending_beams=Beams}) ->
@@ -464,12 +493,12 @@ is_vm_local('localhost',_,true) -> true;
 is_vm_local(_,_,_)              -> false.
 
 set_start_date(undefined)->
-     ts_utils:add_time(now(), ?config(warm_time));
+     ts_utils:add_time(?NOW, ?config(warm_time));
 set_start_date(Date) -> Date.
 
 get_user_param(Client,Config)->
     {ok,IP} = choose_client_ip(Client),
-    {ok, Server} = choose_server(Config#config.servers),
+    {ok, Server} = choose_server(Config#config.servers, Config#config.total_server_weights),
     CPort = choose_port(IP, Config#config.ports_range),
     { {IP, CPort}, Server}.
 
@@ -477,9 +506,8 @@ get_user_param(Client,Config)->
 %% Func: choose_client_ip/1
 %% Args: #client, Dict
 %% Purpose: choose an IP for a client
-%% Returns: {ok, IP, NewDict} IP=IPv4 address {A1,A2,A3,A4}
+%% Returns: {ok, IP, NewDict} IP=IP address
 %%----------------------------------------------------------------------
-%% FIXME: and for IPV6 ?
 choose_client_ip(#client{ip = IPList, host=Host}) ->
     choose_rr(IPList, Host, {0,0,0,0}).
 
@@ -489,8 +517,15 @@ choose_client_ip(#client{ip = IPList, host=Host}) ->
 %% Purpose: choose a server for a new client
 %% Returns: {ok, #server}
 %%----------------------------------------------------------------------
-choose_server(ServerList) ->
-    choose_rr(ServerList, server, #server{}).
+choose_server([Server], _TotalWeight) ->
+    {ok, Server};
+choose_server(Servers, Total) ->
+    choose_server(Servers, random:uniform() * Total, 0).
+
+choose_server([S=#server{weight=P} | _],Rand,Cur) when Rand =< P+Cur->
+    {ok, S};
+choose_server([#server{weight=P} | SList], Rand, Cur) ->
+    choose_server(SList, Rand, Cur+P).
 
 %%----------------------------------------------------------------------
 %% Func: choose_rr/3
@@ -514,25 +549,35 @@ choose_rr(List, Key, _) ->
     {ok, lists:nth(I, List)}.
 
 %%----------------------------------------------------------------------
-%% Func: choose_session/1
+%% Func: choose_session/2
 %% Args: List of #session
 %% Purpose: choose an session randomly
 %% Returns: #session
 %%----------------------------------------------------------------------
-choose_session([Session]) -> %% only one Session
+choose_session([Session], _Total, _PhaseId) -> %% only one Session
     {ok, Session};
-choose_session(Sessions) ->
-    choose_session(Sessions, random:uniform() * 100,0).
+choose_session(Sessions,Total,PhaseId) when is_number(Total)->
+    choose_session(Sessions, random:uniform() * Total, 0, PhaseId);
+choose_session(Sessions,Total,PhaseId) when is_list(Total) ->
+    choose_session(Sessions, random:uniform() * lists:nth(PhaseId, Total), 0, PhaseId).
 
-choose_session([S=#session{popularity=P} | _],Rand,Cur) when Rand =< P+Cur->
+choose_session([S=#session{popularity=P} | _],Rand,Cur,_PhaseId) when is_number(P) andalso Rand =< P+Cur->
     {ok, S};
-choose_session([#session{popularity=P} | SList], Rand, Cur) ->
-    choose_session(SList, Rand, Cur+P).
+choose_session([#session{popularity=P} | SList], Rand, Cur, PhaseId) when is_number(P)->
+    choose_session(SList, Rand, Cur+P, PhaseId);
+choose_session([S=#session{popularity=PopList} | SList],Rand,Cur,PhaseId) ->
+    P = lists:nth(PhaseId,PopList),
+    if
+        Rand =< P+Cur ->
+            {ok, S};
+        true ->
+            choose_session(SList, Rand, Cur+P, PhaseId)
+    end.
 
 
 %%----------------------------------------------------------------------
 %% @spec get_client_cfg(ArrivalPhase::record(arrivalphase),
-%%                      Acc={Total_weight:integer(),Client::record(client),IsLast::binary()}) ->
+%%                      Acc::{Total_weight::integer(),Client::record(client),IsLast::binary()}) ->
 %%                      {{UpdatedPhase::record(arrivalphase),{Intensity::number(),NUsers::integer(),Duration::integer()}}, Acc}
 %% @doc set parameters for given host client and phase.
 %% @end
@@ -545,7 +590,7 @@ get_client_cfg(Arrival=#arrivalphase{duration = Duration,
     ClientIntensity = PhaseIntensity * Weight / TotalWeight,
     NUsers = round(case MaxNumber of
                        infinity -> %% only use the duration to set the number of users
-                           Duration * 1000 * ClientIntensity;
+                           Duration * ClientIntensity;
                        _ ->
                            TmpMax = case {IsLast,CurNumber == MaxNumber} of
                                         {true,_} ->
@@ -555,7 +600,7 @@ get_client_cfg(Arrival=#arrivalphase{duration = Duration,
                                         {false,false} ->
                                             lists:max([1,trunc(MaxNumber * Weight / TotalWeight)])
                                     end,
-                           lists:min([TmpMax, Duration*1000*ClientIntensity])
+                           lists:min([TmpMax, Duration*ClientIntensity])
                    end),
     ?LOGF("New arrival phase ~p for client ~p (last ? ~p): will start ~p users~n",
           [Arrival#arrivalphase.phase,Client#client.host, IsLast,NUsers],?NOTICE),
@@ -591,10 +636,10 @@ print_info() ->
              {value, {_,_ ,V}} -> V;
               _ -> "unknown"
           end,
-    ?LOGF("SYSINFO:Tsung version: ~s~n",[VSN],?NOTICE),
-    ?LOGF("SYSINFO:Erlang version: ~s~n",[erlang:system_info(system_version)],?NOTICE),
-    ?LOGF("SYSINFO:System architecture ~s~n",[erlang:system_info(system_architecture)],?NOTICE),
-    ?LOGF("SYSINFO:Current path: ~s~n",[code:which(tsung)],?NOTICE).
+    ?LOGF("SYSINFO:Tsung version: ~s~n",[VSN],?WARN),
+    ?LOGF("SYSINFO:Erlang version: ~s~n",[erlang:system_info(system_version)],?WARN),
+    ?LOGF("SYSINFO:System architecture ~s~n",[erlang:system_info(system_architecture)],?WARN),
+    ?LOGF("SYSINFO:Current path: ~s~n",[code:which(tsung)],?WARN).
 
 %%----------------------------------------------------------------------
 %% Func: start_file_server/1
@@ -635,15 +680,17 @@ setup_user_servers(FileId,Val) when is_atom(FileId), is_integer(Val) ->
 %% Func: check_config/1
 %% Returns: ok | {error, ErrorList}
 %%----------------------------------------------------------------------
-check_config(Config)->
-    Pop= ts_utils:check_sum(Config#config.sessions, #session.popularity, ?SESSION_POP_ERROR_MSG),
-    %% FIXME: we should not depend on a protocol specific feature here
-    Agents = ts_config_http:check_user_agent_sum(Config#config.session_tab),
-    case lists:filter(fun(X)-> X /= ok  end, [Pop, Agents]) of
-        []        -> ok;
-        ErrorList -> {error, ErrorList}
+check_config(Config=#config{use_weights=UseWeights})->
+    case lists:dropwhile(fun(Pop) -> check_popularity(UseWeights,Pop) == ok end, Config#config.total_popularity) of
+        [] ->
+            ts_config_http:check_user_agent_sum(Config#config.session_tab);
+        [BadPop|_] ->
+            {error, {bad_sum, BadPop, ?SESSION_POP_ERROR_MSG}}
     end.
 
+check_popularity(false, Val) when abs(100-Val) < 0.05 -> ok;
+check_popularity(false,_Val) -> {error, bad_sum };
+check_popularity(true, _Val) -> ok.
 
 load_app(Name) when is_atom(Name) ->
     FName = atom_to_list(Name) ++ ".app",
@@ -665,18 +712,18 @@ load_app(Name) when is_atom(Name) ->
 %% Returns: #config
 %% Purpose: duplicate phases 'load_loop' times.
 %%----------------------------------------------------------------------
-loop_load(Config=#config{load_loop=0,arrivalphases=Arrival}) ->
-    Sorted=lists:keysort(#arrivalphase.phase, Arrival),
-    Config#config{arrivalphases=Sorted};
 loop_load(Config=#config{load_loop=Loop,arrivalphases=Arrival}) when is_integer(Loop) ->
-    loop_load(Config, ts_utils:keymax(#arrivalphase.phase, Arrival), Arrival ).
+    Sorted=lists:keysort(#arrivalphase.phase, Arrival),
+    {SortedWithId,_} = lists:mapfoldl(fun(Phase, Id) -> {Phase#arrivalphase{id=Id}, Id+1} end, 1, Sorted),
+    loop_load(Config#config{arrivalphases=SortedWithId}, ts_utils:keymax(#arrivalphase.phase, Arrival), SortedWithId ).
 
 %% We have a list of n phases: duplicate the list and increase by the
 %% max to get a new unique id for all phases. Here we don't care about
 %% the order, so we start with the last iteration (Loop* Max)
 loop_load(Config=#config{load_loop=0},_,Current) ->
-    Sorted=lists:keysort(#arrivalphase.phase, Current),% is it necessary ?
-    Config#config{arrivalphases = Sorted};
+    Sorted=lists:keysort(#arrivalphase.phase, Current),
+    ?LOGF("sorted phases: ~p ~n", [Sorted], ?DEB),
+    Config#config{arrivalphases=Sorted};
 loop_load(Config=#config{load_loop=Loop, arrivalphases=Arrival},Max,Current) ->
     Fun= fun(Phase) -> Phase+Max*Loop end,
     NewArrival = lists:keymap(Fun,#arrivalphase.phase,Arrival),
@@ -685,29 +732,55 @@ loop_load(Config=#config{load_loop=Loop, arrivalphases=Arrival},Max,Current) ->
 %% @doc sort static users by start time
 sort_static(Config=#config{static_users=S})->
     ?LOGF("sort static users: ~p ~n", [S], ?DEB),
-    SortedL= lists:keysort(1,S),
+    ES = expand_static(S,Config#config.sessions),
+    SortedL= lists:keysort(1,ES),
     Config#config{static_users=static_name_to_session(Config#config.sessions,SortedL)}.
 
+%% expand static users (if it contains wildcards)
+expand_static(StaticUsers, Sessions) ->
+    Names = lists:map(fun(#session{name=A}) -> A end ,Sessions),
+    expand_static(StaticUsers, Names, []).
+
+expand_static([], _Names, Static) -> Static;
+expand_static([{Delay, Name} | Static],SessionsNames, Acc) ->
+    Names  = ts_utils:wildcard(Name, SessionsNames),
+    NewStatic = lists:map(fun(N) -> {Delay, N} end, Names),
+    expand_static(Static, SessionsNames, Acc ++ NewStatic).
+
+
 %%
 %% @doc start a remote beam
 %%
 start_slave(Host, Name, Args) when is_atom(Host), is_atom(Name)->
     case slave:start(Host, Name, Args) of
         {ok, Node} ->
-            ?LOGF("started newbeam on node ~p ~n", [Node], ?NOTICE),
+            ?LOGF("Remote beam started on node ~p ~n", [Node], ?NOTICE),
             Res = net_adm:ping(Node),
-            ?LOGF("ping ~p ~p~n", [Node,Res], ?NOTICE),
+            ?LOGF("ping ~p ~p~n", [Node,Res], ?INFO),
             Node;
         {error, Reason} ->
             ?LOGF("Can't start newbeam on host ~p (reason: ~p) ! Aborting!~n",[Host, Reason],?EMERG),
             exit({slave_failure, Reason})
     end.
+choose_port(_,_, undefined) ->
+    {[],0};
+choose_port(Client,undefined, Range) ->
+    choose_port(Client,dict:new(), Range);
+choose_port(ClientIp,Ports, {Min, Max}) ->
+    case dict:find(ClientIp,Ports) of
+        {ok, Val} when Val =< Max ->
+            NewPorts=dict:update_counter(ClientIp,1,Ports),
+            {NewPorts,Val};
+        _ -> % Max Reached or new entry
+            NewPorts=dict:store(ClientIp,Min+1,Ports),
+            {NewPorts,Min}
+    end.
 
 choose_port(_,undefined) -> 0;
 choose_port(_, _Range)   -> -1.
 
-%% @spec session_name_to_session(Sessions::list(), Static::list() ) -> StaticUsers::list()
-%% @doc convert session name to session id in static users list
+%% @spec static_name_to_session(Sessions::list(), Static::list() ) -> StaticUsers::list()
+%% @doc convert session name to session id in static users list @end
 static_name_to_session(Sessions, Static) ->
     ?LOGF("Static users with session id ~p~n",[Static],?DEB),
     Search = fun({Delay,Name})->
@@ -737,7 +810,7 @@ set_nodename(NodeId) when is_integer(NodeId)->
 set_max_duration(0) -> ok; % nothing to do
 set_max_duration(Duration) when Duration =< 4294967 ->
     ?LOGF("Set max duration of test: ~p s ~n",[Duration],?NOTICE),
-    erlang:start_timer(Duration*1000, self(), end_tsung ).
+    erlang:start_timer((Duration+?config(warm_time))*1000, self(), end_tsung ).
 
 
 
@@ -760,6 +833,13 @@ local_launcher([Host],LogDir,Config) ->
         ok ->
             ?LOG("Application started, activate launcher, ~n", ?INFO),
             application:set_env(tsung, debug_level, Config#config.loglevel),
+            case Config#config.ports_range of
+                {Min, Max} ->
+                    application:set_env(tsung, cport_min, Min),
+                    application:set_env(tsung, cport_max, Max);
+                undefined ->
+                    ""
+            end,
             ts_launcher_static:launch({node(), Host, []}),
             ts_launcher:launch({node(), Host, [], Config#config.seed}),
             1 ;
@@ -774,19 +854,13 @@ remote_launcher(Host, NodeId, Args) when is_list(NodeId)->
     remote_launcher(Host, list_to_integer(NodeId), Args);
 remote_launcher(Host, NodeId, Args)->
     Name = set_nodename(NodeId),
-    ?LOGF("starting newbeam on host ~p with Args ~p~n", [Host, Args], ?INFO),
+    ?LOGF("starting newbeam ~p on host ~p with Args ~p~n", [Name, Host, Args], ?INFO),
     start_slave(Host, Name, Args).
 
 set_remote_args(LogDir,PortsRange)->
-    {ok, [[BootController]]}    = init:get_argument(boot),
-    ?DebugF("BootController ~p~n", [BootController]),
-    {ok, [[?TSUNGPATH,PathVar]]}    = init:get_argument(boot_var),
-    ?DebugF("BootPathVar ~p~n", [PathVar]),
     {ok, PAList}    = init:get_argument(pa),
     PA = lists:flatmap(fun(A) -> [" -pa "] ++A end,PAList),
     ?DebugF("PA list ~p ~n", [PA]),
-    Boot = re:replace(BootController,"tsung_controller","tsung",[{return,list},global]),
-    ?DebugF("Boot ~p~n", [Boot]),
     Sys_Args= ts_utils:erl_system_args(),
     LogDirEnc = encode_filename(LogDir),
     Ports = case PortsRange of
@@ -795,8 +869,8 @@ set_remote_args(LogDir,PortsRange)->
                 undefined ->
                     ""
             end,
-    lists:flatten([ Sys_Args," -boot ", Boot,
-                    " -boot_var ", ?TSUNGPATH, " ",PathVar, PA,
+    lists:flatten([ Sys_Args,
+                    PA,
                     " +K true ",
                     " -tsung debug_level ", integer_to_list(?config(debug_level)),
                     " -tsung log_file ", LogDirEnc, Ports
@@ -808,6 +882,9 @@ set_remote_args(LogDir,PortsRange)->
 %%      single node per host
 %% @end
 
+get_one_node_per_host([]) ->
+    %%no remote nodes, we are using a controller vm
+    [node()];
 get_one_node_per_host(RemoteNodes) ->
     get_one_node_per_host(RemoteNodes,dict:new()) .
 
@@ -823,3 +900,64 @@ get_one_node_per_host([Node | Nodes], Dict) ->
             NewDict = dict:store(Host, Node, Dict),
             get_one_node_per_host(Nodes,NewDict)
     end.
+
+
+%% compute popularities of sessions for all phases
+compute_popularities(Config=#config{arrivalphases=Phases, sessions=Sessions}) ->
+
+    %% popularities can contains wildcards, need to expand them
+    Names = lists:map(fun(#session{name=A}) -> A end ,Sessions),
+    Expand = fun( Phase = #arrivalphase{popularities= Pops} ) ->
+                     NewPops = lists:foldl(fun({Name,Popularity},Acc) ->
+                                                   Expanded = ts_utils:wildcard(Name, Names),
+                                                   Acc ++ lists:map(fun(X) -> {X, Popularity} end, Expanded)
+                                           end, [], Pops),
+                     Phase#arrivalphase{popularities=NewPops}
+             end,
+    NewPhases = lists:map(Expand,Phases),
+    ?LOGF("Compute popularities per phases ~p",[NewPhases],?DEB),
+
+    F = fun(Session=#session{popularity=Pop, name=Name}) ->
+           NewPop = set_pop(Name, Pop, NewPhases),
+           Session#session{popularity=NewPop}
+        end,
+    NewSessions = lists:map(F, Sessions),
+    ?LOGF("Old sessions:~p",[Sessions],?DEB),
+    ?LOGF("New sessions:~p",[NewSessions],?DEB),
+    Config#config{sessions=NewSessions, arrivalphases = NewPhases, total_popularity=update_total_pop(Config#config.use_weights, NewPhases, NewSessions)}.
+
+update_total_pop(UseWeight,Phases, Sessions) ->
+    update_total_pop(UseWeight, length(Phases), Sessions, []).
+
+update_total_pop(_UseWeight,0, _, Total) ->
+    ?LOGF("New Total popularities:~p",[Total],?DEB),
+    Total;
+update_total_pop(UseWeight,N, Sessions, Total)   ->
+    Sum = fun(#session{popularity=P},Acc) when is_number(P) ->
+                  Acc+P ;
+             (#session{popularity=L},Acc) ->
+                  Acc+lists:nth(N,L)
+          end,
+    PhaseTotal = lists:foldl(Sum, 0, Sessions),
+    update_total_pop(UseWeight, N-1, Sessions, [PhaseTotal |Total]).
+
+%% set popularity of session 'Name' per phase (needed when <session_setup> is used)
+set_pop(Name,Popularity,Phases) ->
+    set_pop(Name,Popularity,Phases,[]).
+
+set_pop(_Name,_Popularity,[], Acc) ->
+    %% optimization: if all values are equal, return a single value and not a list
+    Min=lists:min(Acc),
+    case lists:max(Acc) of
+        Min -> Min; % min = max
+        _   -> lists:reverse(Acc)
+    end;
+set_pop(Name,Popularity,[#arrivalphase{popularities=Pop}|Tail], Acc) ->
+    New = case lists:keysearch(Name,1,Pop) of
+              false ->
+                  Popularity;
+              {value, {_, Val}} ->
+                  Val
+          end,
+    set_pop(Name,Popularity,Tail, [New|Acc]).
+
