@@ -21,13 +21,14 @@
 %%%  In addition, as a special exception, you have the permission to
 %%%  link the code of this program with any library released under
 %%%  the EPL license and distribute linked combinations including
-%%%  the two.
+%%%  the two; the MPL (Mozilla Public License), which EPL (Erlang
+%%%  Public License) is based on, is included in this exception.
 
 -module(ts_proxy_http).
 -vc('$Id$ ').
 -author('Nicolas.Niclausse@niclux.org').
 
--include("ts_profile.hrl").
+-include("ts_macros.hrl").
 -include("ts_http.hrl").
 -include("ts_recorder.hrl").
 
@@ -334,6 +335,11 @@ record_request(State=#state_rec{prev_host=Host, prev_port=Port, prev_scheme=Sche
 %% should we save the content of a  POST in an external binary file ?
 save_binary_post("multipart/form-data"++_Tail) -> true;
 save_binary_post("application/x-amf") -> true;
+save_binary_post("application/x-silverlight-app") -> true;
+save_binary_post("application/xaml+xml") -> true;
+save_binary_post("application/x-ms-xbap") -> true;
+save_binary_post("application/soap+msbin1") -> true;
+save_binary_post("application/msbin1") -> true;
 save_binary_post(_) -> false.
 
 %%--------------------------------------------------------------------
